-- kl_balloons.lua

--[[
    Hi I'm Namolos. I like to make terrible things in short scripts
    
    This nightmare of a script replaces all random item boxes with balloons. Balloons now act as random items, as well
    as retaining the bouncy behavior so many people utterly despise
    
    And with thanks to FabulousNinji for the sprite, egg balloons are now a thing as well. Hope you have your sniping
    skills down because it's time to literally bounce people around!
    
    Feel free to use this in other projects or as a reference
]]--

-- regular old item balloon
addHook("MobjThinker", function(mobj)
	if (mobj.balloon == nil)
	    mobj.balloon = P_SpawnMobj(mobj.x, mobj.y, mobj.z+18*FRACUNIT, MT_BALLOON);
	    mobj.balloon.color = P_RandomKey(MAXSKINCOLORS-1)+1;
	end
	--mobj.flags2 = $|MF2_DONTDRAW;  -- remnants of forgotten time
	P_RemoveMobj(mobj);
end, MT_RANDOMITEM)


-- incredibly devilish egg balloon
freeslot("SPR_EBLN")
local function eggballoon(mobj)
    local zfactor = mobj.type == MT_EGGMANITEM and 18*FRACUNIT or 0;
    local trans = mobj.type == MT_EGGMANITEM_SHIELD and TR_TRANS20 or 0;
    local color = P_RandomKey(MAXSKINCOLORS-1)+1;
    if (mobj.balloon == nil)
	    mobj.balloon = P_SpawnMobj(mobj.x, mobj.y, mobj.z+zfactor, MT_BALLOON);
	    mobj.balloon.color = (color ~= SKINCOLOR_BLACK and color) or SKINCOLOR_WHITE;  -- omit black for egg balloons. Not good
	    mobj.balloon.eggballoon = 1;
	    mobj.balloon.parent = mobj;  -- keep this ID... gonna invert things to hopefully make this work more consistently
	    mobj.balloon.target = mobj.target;
	    mobj.balloon.threshold = mobj.threshold;
	end
	mobj.flags2 = $|MF2_DONTDRAW;
	
	-- as long as the egg monitor exists, we're gonna keep the balloon attached to it
	if (mobj.balloon.valid)
	    P_TeleportMove(mobj.balloon, mobj.x, mobj.y, mobj.z+zfactor);
	    mobj.balloon.sprite = SPR_EBLN;
	    mobj.balloon.frame = trans|$;
	    mobj.balloon.threshold = mobj.threshold;
	    mobj.balloon.scale = mobj.scale;
	end
end
addHook("MobjThinker", eggballoon, MT_EGGMANITEM_SHIELD)
addHook("MobjThinker", eggballoon, MT_EGGMANITEM)
-- Extra bit, delete the leftover egg balloon if dropped/tossed
addHook("MobjRemoved", function(mobj)
    if (mobj.balloon and mobj.balloon.valid)
        P_RemoveMobj(mobj.balloon);
    end
end, MT_EGGMANITEM_SHIELD)
-- We've been popped. Get ready to cease to exist! (This only applies to egg balloons. Regular balloons regenerate normally)
addHook("MobjThinker", function(mobj)
	if (mobj.eggballoon ~= nil) and (mobj.eggballoon > 1)
	    if (mobj.parent and mobj.parent.valid)
	        P_RemoveMobj(mobj.parent);
	    end
	    if (mobj and mobj.valid)
	        P_RemoveMobj(mobj);
	    end
	end
end, MT_BALLOON)

-- eggman box itself won't have collision anymore
addHook("TouchSpecial", function(mobj, t)
    return true;
end, MT_EGGMANITEM)
addHook("TouchSpecial", function(mobj, t)
    return true;
end, MT_EGGMANITEM_SHIELD)


-- fun fact for anyone reading this. Originally, I had just made item boxes invisible and spawned balloons on top of them.
-- However, this resulted in bounce chains leaving invisible boxes behind... which would let some players get an item without
-- suffering. So instead, I removed them and made balloons behave like item boxes. Now everyone has to suffer!
addHook("TouchSpecial", function(mobj, t)
    if (mobj.eggballoon == nil)  -- regular balloon
        if ( (t.player.kartstuff[k_itemroulette] == 0) and (t.player.kartstuff[k_itemamount] == 0)
         and (t.player.kartstuff[k_growshrinktimer] < 1) and (t.player.kartstuff[k_rocketsneakertimer] == 0) and (t.player.kartstuff[k_eggmanexplode] == 0) )  -- because these don't hold the item box hostage for some reason?
            t.player.kartstuff[k_itemroulette] = 1;
        end
    else                         -- egg balloon
        if (mobj.target == t) and (mobj.threshold > 0)  -- no hurting the player while they're still holding it!
           return true;
        end

        if ( (t.player.kartstuff[k_growshrinktimer] < 1) and (t.player.kartstuff[k_invincibilitytimer] == 0) and (t.player.kartstuff[k_hyudorotimer] == 0) and (t.player.kartstuff[k_spinouttimer] == 0) and (t.player.powers[pw_flashing] == 0) -- Immune in these states
         and (t.player.kartstuff[k_roulettetype] ~= 2) and (t.player.kartstuff[k_eggmanexplode] == 0) )
            K_DropItems(t.player);
            t.player.kartstuff[k_itemroulette] = 1;
            t.player.kartstuff[k_roulettetype] = 2;
            if (mobj.parent and mobj.parent.valid and mobj.parent.target and mobj.parent.target.valid)  -- because for some reason this doesn't always get set correctly on both ends? fuck idk but this should stop this function from halting early and let us actually get rid of the balloon
                t.player.kartstuff[k_eggmanblame] = #mobj.parent.target.player;
            elseif (mobj and mobj.valid and mobj.target and mobj.target.valid)
                t.player.kartstuff[k_eggmanblame] = #mobj.target.player;
            else
                t.player.kartstuff[k_eggmanblame] = -1;
            end
        end
        mobj.eggballoon = 2;  -- cheesy me, this'll be the 'popped' state as well :shitsfree:
    end
end, MT_BALLOON)
